==> andy-audio-0.2.0.tgz: package/src/tools/index.ts <==
// Very rough usage of Andy to load an play sounds.
// You'll want implement this into something more robust
// like an react context or something along those lines.
// 
// This shows a bunch of random stuff and should not be used as is.

import * as Andy from '../andy';

import { GeneralMidiInstruments, Patch, SoundOn } from '../midi/types';
import { loadMidi } from '../midi/song';
import { AndyPatch } from '../midi/patchBank';

////////////////////////////////////////////////////////////
import { MOD } from '../mod/song';
import { dataBase64 } from '../../static/test_mod_b64'
import { loadMod, samplePlayer } from '../mod/audio';
////////////////////////////////////////////////////////////
import * as MODPlayer from '../mod/player';
import * as MIDIPlayer from '../midi/player';
////////////////////////////////////////////////////////////

declare var window: any;

type AllSounds = {
    s0: Promise<Andy.Sound> | Andy.Sound,
    s1: Promise<Andy.Sound> | Andy.Sound,
    s2: Promise<Andy.Sound> | Andy.Sound,
    s3: Promise<Andy.Sound> | Andy.Sound,
    s4: Promise<Andy.Sound> | Andy.Sound,
    s5: Promise<Andy.Sound> | Andy.Sound,
}

const system = new Andy.SoundSystem();
const sounds: AllSounds = {
    s0: Andy.loadSound("./static/coin4.wav", system),
    s1: Andy.loadSound("./static/humm.wav", system),
    s2: Andy.loadSound("./static/whatisthis.wav", system),
    s3: Andy.loadSound("./static/icon.wav", system),
    s4: Andy.loadSound("./static/doortrek2.wav", system),
    s5: Andy.loadSound("./static/There+is+no+there+out+there.mp3", system)
};

window['initSoundSystem'] = async () => {
    const s = await Promise.all([sounds.s0, sounds.s1, sounds.s2, sounds.s3, sounds.s4, sounds.s5]);
    sounds.s0 = s[0];
    sounds.s1 = s[1];
    sounds.s2 = s[2];
    sounds.s3 = s[3];
    sounds.s4 = s[4];
    sounds.s5 = s[5];
    window['soundSystem'] = system;
    window['sounds'] = sounds;
}

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////
// Fake Audio player
let songRun: Andy.MidiRun;

// @ts-ignore
document.getElementById("playpause").addEventListener('click', () => {
    Andy.playHeadToggle();
    Andy.silenceRun(songRun);
});

// @ts-ignore
document.getElementById("rewind").addEventListener('click', () => {
    Andy.playHeadRewind();
    Andy.silenceRun(songRun);
    Andy.rewindRun(songRun);
});

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////
// Fake keyboard
const myPatch: Patch = {
    len: 2,
    vca: { gain: 1 },
    voices: [
        {
            vco: { type: 'sine', freqOffset: 0, detune: 0 },
            vca: { gainOffset: 0 },
        },
        {
            vco: { type: 'sine', freqOffset: 65, detune: 0 },
            vca: { gainOffset: -0.29 },
        }
    ],
    compressor: {
        threshold: -1,
        knee: 40,
        ratio: 12,
        attack: 0,
        release: .25,
    },
    adsr: {
        attack: 0,
        decay: 0,
        sustain: 1,
        release: 1,
    },
    filter: {
        type: 'highpass',
        frequency: 300,
        q: 1.04,
        gain: 1,
    }
};
const a = Andy.patchPlayer(system, myPatch);
const s1 = Andy.patchPlayer(system, AndyPatch.get(GeneralMidiInstruments.AcousticBass) ?? myPatch);
const kick = Andy.patchPlayer(system, AndyPatch.get(GeneralMidiInstruments.TaikoDrum) ?? myPatch);
const s2 = Andy.patchPlayer(system, AndyPatch.get(GeneralMidiInstruments.ChurchOrgan) ?? myPatch);


///////////////////////////////////////////////////////
//////
// Loading a song
// @ts-ignore
const arrayBuffer = Uint8Array.from(window.atob(dataBase64), c => c.charCodeAt(0));
let mod = new MOD(arrayBuffer.buffer);
const modSong = mod.song();

// loadMod("static/a12.mod").then((song) => {
//     const songRun = MODPlayer.createRun(system, song);
//     MODPlayer.playRun(system, songRun);
// });

loadMidi("static/TestMidiTuneTempo.mid").then((song) => {
    songRun = MIDIPlayer.createRun(system, song);
    MIDIPlayer.playRun(system, songRun);
});

///////////////////////////////////////////////////////

// loading samples from a MOD song
const sam = samplePlayer(system, modSong, 0x0E);
const samloo = samplePlayer(system, modSong, 0x0F);

///////////////////////////////////////
///////////////////////////////////////
///////////////////////////////////////
// Testing keyboard

const keyStatus: any = {};
const n = s2;
document.addEventListener('keydown', function (e: KeyboardEvent) {
    const speakers = system.masterGainNode();

    switch (e.key) {
        case 'z':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("C-4", system.currentTime(), speakers);
            break;
        case 's':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("C#4", system.currentTime(), speakers);
            break;
        case 'x':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("D-4", system.currentTime(), speakers);
            break;
        case 'd':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("D#4", system.currentTime(), speakers);
            break;
        case 'c':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("E-4", system.currentTime(), speakers);
            break;
        case 'v':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("F-4", system.currentTime(), speakers);
            break;
        case 'g':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("F#4", system.currentTime(), speakers);
            break;
        case 'b':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("G-4", system.currentTime(), speakers);
            break;
        case 'h':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("G#4", system.currentTime(), speakers);
            break;
        case 'n':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("A-4", system.currentTime(), speakers);
            break;
        case 'j':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("A#4", system.currentTime(), speakers);
            break;
        case 'm':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("B-4", system.currentTime(), speakers);
            break;
        case ',':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("C-5", system.currentTime(), speakers);
            break;
        ///////////////////////////////////////
        case 'q':
            if (!keyStatus[e.key]) keyStatus[e.key] = sam("C-4", system.currentTime(), speakers);
            break;
        case 'a':
            if (!keyStatus[e.key]) keyStatus[e.key] = samloo("C-4", system.currentTime(), speakers);
            break;
    }
});

document.addEventListener('keyup', function (e: KeyboardEvent) {
    if (keyStatus[e.key]) {
        keyStatus[e.key]();
        delete keyStatus[e.key];
    }
});

==> andy-audio-0.3.2.tgz: package/src/tools/index.ts <==
// Very rough usage of Andy to load an play sounds.
// You'll want implement this into something more robust
// like an react context or something along those lines.
// 
// This shows a bunch of random stuff and should not be used as is.

import * as Andy from '../andy';

import { GeneralMidiInstruments, Patch } from '../midi/types';
import { loadMidi } from '../midi/song';
import { AndyPatch } from '../midi/patchBank';

////////////////////////////////////////////////////////////
import { MOD } from '../mod/song';
import { dataBase64 } from '../../static/test_mod_b64'
import { loadMod, samplePlayer } from '../mod/audio';
////////////////////////////////////////////////////////////
// import * as MODPlayer from '../mod/player';
// import * as MIDIPlayer from '../midi/player';
import { createRun, createTempo, insertRun, MidiRun, PerTickCallback, playHeadRewind, playHeadToggle, rewindRun, silenceRun, toSignature } from '../andy';
////////////////////////////////////////////////////////////

declare var window: any;

type AllSounds = {
    s0: Promise<Andy.Sound> | Andy.Sound,
    s1: Promise<Andy.Sound> | Andy.Sound,
    s2: Promise<Andy.Sound> | Andy.Sound,
    s3: Promise<Andy.Sound> | Andy.Sound,
    s4: Promise<Andy.Sound> | Andy.Sound,
    s5: Promise<Andy.Sound> | Andy.Sound,
}

const system = new Andy.SoundSystem();
const sounds: AllSounds = {
    s0: Andy.loadSound("./static/coin4.wav", system),
    s1: Andy.loadSound("./static/humm.wav", system),
    s2: Andy.loadSound("./static/whatisthis.wav", system),
    s3: Andy.loadSound("./static/icon.wav", system),
    s4: Andy.loadSound("./static/doortrek2.wav", system),
    s5: Andy.loadSound("./static/There+is+no+there+out+there.mp3", system)
};

window['initSoundSystem'] = async () => {
    const s = await Promise.all([sounds.s0, sounds.s1, sounds.s2, sounds.s3, sounds.s4, sounds.s5]);
    sounds.s0 = s[0];
    sounds.s1 = s[1];
    sounds.s2 = s[2];
    sounds.s3 = s[3];
    sounds.s4 = s[4];
    sounds.s5 = s[5];
    window['soundSystem'] = system;
    window['sounds'] = sounds;
}

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////
// Fake Audio player
let songRun: MidiRun;

// @ts-ignore
document.getElementById("playpause").addEventListener('click', () => {
    playHeadToggle();
    silenceRun(songRun);
});

// @ts-ignore
document.getElementById("rewind").addEventListener('click', () => {
    silenceRun(songRun);
    playHeadRewind();
    rewindRun(songRun);
});

//////////////////////////////////////////////
//////////////////////////////////////////////
//////////////////////////////////////////////
// Fake keyboard
const myPatch: Patch = {
    len: 2,
    vca: { gain: 1 },
    voices: [
        {
            vco: { type: 'sine', freqOffset: 0, detune: 0 },
            vca: { gainOffset: 0 },
        },
        {
            vco: { type: 'sine', freqOffset: 65, detune: 0 },
            vca: { gainOffset: -0.29 },
        }
    ],
    compressor: {
        threshold: -1,
        knee: 40,
        ratio: 12,
        attack: 0,
        release: .25,
    },
    adsr: {
        attack: 0,
        decay: 0,
        sustain: 1,
        release: 1,
    },
    filter: {
        type: 'highpass',
        frequency: 300,
        q: 1.04,
        gain: 1,
    }
};
const a = Andy.patchPlayer(system, myPatch);
const s1 = Andy.patchPlayer(system, AndyPatch.get(GeneralMidiInstruments.AcousticBass) ?? myPatch);
const kick = Andy.patchPlayer(system, AndyPatch.get(GeneralMidiInstruments.TaikoDrum) ?? myPatch);
const s2 = Andy.patchPlayer(system, AndyPatch.get(GeneralMidiInstruments.ChurchOrgan) ?? myPatch);

///////////////////////////////////////////////////////
//////
// Loading a song
// @ts-ignore
const arrayBuffer = Uint8Array.from(window.atob(dataBase64), c => c.charCodeAt(0));
let mod = new MOD(arrayBuffer.buffer);
const modSong = mod.song();

// To show the song playback time in the UI
function timeDisplayCallback(): PerTickCallback {
    const tempo = createTempo(480, 500000);
    const sig = toSignature("4/4");

    const timeDom = document.getElementById("musictime") as HTMLDivElement;
    if (timeDom) timeDom.innerHTML = `0.0.0`;

    const tickCallBack: PerTickCallback = (tick, time, bi) => {
        if (timeDom) {
            timeDom.innerHTML = `${bi.measure}.${bi.beat}.${('00' + bi.tick).slice(-3)} :: ${tick}`
        }
        return [tempo, sig];
    }
    return tickCallBack
}

// Example of setting up a callback to loop to markers 
// setup as "LoopEnd" and "LoopStart"
function markerLoopCallback(song: Andy.MidiRun): PerTickCallback {
    const loopTrigger = song.song.markers.get("LoopEnd");
    const loopRewind = song.song.markers.get("LoopStart");
    return (tick, _time, _bi) => {
        if (tick === loopTrigger.abstime) {
            Andy.setRunTimeTo(loopRewind.abstime, songRun);
            Andy.playHeadTo(loopRewind.abstime);
        }
        return [createTempo(480, 500000), toSignature("4/4")];
    }
}

////////

// loadMod("static/TestChipTune.mod").then((song) => {
//     const songRun = MODPlayer.createRun(system, song);
//     songRun.tickListeners.push(timeDisplayCallback());
//     MODPlayer.insertRun(system, songRun);
// });

loadMidi("static/TestMidiTuneMarkers.mid").then((song) => {
    // Create a run from the midi song data we loaded
    songRun = createRun(system, song);
    // Add our own tick callback for playback display
    songRun.tickListeners.push(timeDisplayCallback());
    // Add a callback to jump around with the loops
    songRun.tickListeners.push(markerLoopCallback(songRun));

    // Insert the run into the player
    insertRun(system, songRun);
    // Now to play, toggle the play head
});

///////////////////////////////////////////////////////

// loading samples from a MOD song
const sam = samplePlayer(system, modSong, 0x0E);
const samloo = samplePlayer(system, modSong, 0x0F);

///////////////////////////////////////
///////////////////////////////////////
///////////////////////////////////////
// Testing keyboard

const keyStatus: any = {};
const n = s2;
document.addEventListener('keydown', function (e: KeyboardEvent) {
    const speakers = system.masterGainNode();

    switch (e.key) {
        case 'z':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("C-4", system.currentTime(), speakers);
            break;
        case 's':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("C#4", system.currentTime(), speakers);
            break;
        case 'x':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("D-4", system.currentTime(), speakers);
            break;
        case 'd':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("D#4", system.currentTime(), speakers);
            break;
        case 'c':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("E-4", system.currentTime(), speakers);
            break;
        case 'v':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("F-4", system.currentTime(), speakers);
            break;
        case 'g':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("F#4", system.currentTime(), speakers);
            break;
        case 'b':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("G-4", system.currentTime(), speakers);
            break;
        case 'h':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("G#4", system.currentTime(), speakers);
            break;
        case 'n':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("A-4", system.currentTime(), speakers);
            break;
        case 'j':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("A#4", system.currentTime(), speakers);
            break;
        case 'm':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("B-4", system.currentTime(), speakers);
            break;
        case ',':
            if (!keyStatus[e.key]) keyStatus[e.key] = n("C-5", system.currentTime(), speakers);
            break;
        ///////////////////////////////////////
        case 'q':
            if (!keyStatus[e.key]) keyStatus[e.key] = sam("C-4", system.currentTime(), speakers);
            break;
        case 'a':
            if (!keyStatus[e.key]) keyStatus[e.key] = samloo("C-4", system.currentTime(), speakers);
            break;
    }
});

document.addEventListener('keyup', function (e: KeyboardEvent) {
    if (keyStatus[e.key]) {
        keyStatus[e.key]();
        delete keyStatus[e.key];
    }
});
